@@ -867,4 +867,161 @@ TEST_CASE("Github #5008: bad tautomers for phosphorous compounds") {
     REQUIRE(ct);
     CHECK(MolToSmiles(*ct) == "C[PH](C)=O");
   }
-}
\ No newline at end of file
+}
+
+TEST_CASE("Github #5169: Standardization via RDKit breaks molecules",
+          "[uncharger]") {
+  SECTION("basics") {
+    SmilesParserParams ps;
+    ps.sanitize = false;
+    std::vector<std::string> smis = {"C[O+](C)C", "[H]/[O+]=C/Cl"};
+    for (const auto &smi : smis) {
+      std::unique_ptr<RWMol> m{SmilesToMol(smi, ps)};
+      REQUIRE(m);
+      m->updatePropertyCache(false);
+      MolStandardize::Uncharger uncharger;
+      std::unique_ptr<ROMol> outm(uncharger.uncharge(*m));
+      REQUIRE(outm);
+      INFO("failing for smiles " << smi);
+      CHECK(outm->getAtomWithIdx(1)->getFormalCharge() == 1);
+    }
+  }
+}
+
+TEST_CASE("asymmetric imine tautomer generation", "[tautomers]") {
+  SECTION("basics") {
+    MolStandardize::TautomerEnumerator tenum;
+    // clang-format off
+    std::vector<std::pair<std::string, unsigned>> data = {
+        {"C=C1NNC(=O)N1*", 2},
+        {"CC1=NN=C(O)N1*", 2},
+        {"C-C=NC", 1},
+        {"C-C=N", 2},
+        {"C-C=Nc1ccccc1", 2},
+    };
+    // clang-format on
+    for (const auto &pr : data) {
+      INFO(pr.first);
+      std::unique_ptr<RWMol> m(SmilesToMol(pr.first));
+      auto res = tenum.enumerate(*m);
+      CHECK(res.size() == pr.second);
+    }
+  }
+}
+
+TEST_CASE("Github 5317: standardization failing with zwitterionic sulfone") {
+  SECTION("basics") {
+    auto m = "C[S+2]([O-])([O-])C([O-])C(=O)O"_smiles;
+    REQUIRE(m);
+    MolStandardize::Uncharger uc;
+    std::unique_ptr<ROMol> res{uc.uncharge(*m)};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "C[S+2]([O-])([O-])C(O)C(=O)O");
+  }
+  SECTION("don't overdo it") {
+    auto m = "C[S+2]([O-])([O-])C([O-])C(=O)O.[Na+]"_smiles;
+    REQUIRE(m);
+    MolStandardize::Uncharger uc;
+    std::unique_ptr<ROMol> res{uc.uncharge(*m)};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "C[S+2]([O-])([O-])C([O-])C(=O)O.[Na+]");
+  }
+}
+
+TEST_CASE("Github 5318: standardizing unsanitized molecules should work") {
+  SmilesParserParams ps;
+  ps.sanitize = false;
+  ps.removeHs = false;
+  std::unique_ptr<RWMol> m{SmilesToMol("C[S+2]([O-])([O-])C([O-])C(=O)O", ps)};
+  REQUIRE(m);
+  std::unique_ptr<RWMol> m2{SmilesToMol("Cc1[nH]ncc1.[Cl]", ps)};
+  REQUIRE(m2);
+  SECTION("reionizer") {
+    MolStandardize::Reionizer reion;
+    std::unique_ptr<ROMol> res{reion.reionize(*m)};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "C[S+2]([O-])([O-])C(O)C(=O)[O-]");
+  }
+  SECTION("uncharger") {
+    MolStandardize::Uncharger uc;
+    std::unique_ptr<ROMol> res{uc.uncharge(*m)};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "C[S+2]([O-])([O-])C(O)C(=O)O");
+  }
+  SECTION("normalizer") {
+    std::unique_ptr<ROMol> res{MolStandardize::normalize(m.get())};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "CS(=O)(=O)C([O-])C(=O)O");
+  }
+  SECTION("tautomer") {
+    std::unique_ptr<ROMol> res{MolStandardize::canonicalTautomer(m2.get())};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "Cc1cc[nH]n1.Cl");
+  }
+  SECTION("fragments") {
+    std::unique_ptr<ROMol> res{MolStandardize::removeFragments(m2.get())};
+    REQUIRE(res);
+    CHECK(MolToSmiles(*res) == "Cc1ccn[nH]1");
+  }
+}
+
+TEST_CASE("Github #5320: cleanup() and stereochemistry") {
+  SECTION("basics") {
+    auto m = "Cl[C@](O)([O-])C(=O)O"_smiles;
+    REQUIRE(m);
+    CHECK(m->getAtomWithIdx(1)->getChiralTag() !=
+          Atom::ChiralType::CHI_UNSPECIFIED);
+    std::unique_ptr<RWMol> m2{MolStandardize::cleanup(m.get())};
+    REQUIRE(m2);
+    CHECK(m2->getAtomWithIdx(1)->getChiralTag() ==
+          Atom::ChiralType::CHI_UNSPECIFIED);
+    CHECK(MolToSmiles(*m2) == "O=C([O-])C(O)(O)Cl");
+  }
+}
+
+TEST_CASE("Github #5402: order dependence of tautomer transforms") {
+  SECTION("as-reported") {
+    MolStandardize::TautomerEnumerator te;
+
+    auto m1 = "c1ccc([C@@H](CC2=NCCN2)c2ccccn2)cc1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "C([C@H](C1=CC=CC=C1)C2=NC=CC=C2)C3=NCCN3"_smiles;
+    REQUIRE(m2);
+    std::cerr << " * - * - * - * m1" << std::endl;
+    std::unique_ptr<ROMol> res1{te.canonicalize(*m1)};
+    REQUIRE(res1);
+    std::cerr << " * - * - * - * m2" << std::endl;
+    std::unique_ptr<ROMol> res2{te.canonicalize(*m2)};
+    REQUIRE(res2);
+    CHECK(MolToSmiles(*res1) == MolToSmiles(*res2));
+  }
+  SECTION("zoom") {
+    MolStandardize::CleanupParameters params;
+    const std::vector<
+        std::tuple<std::string, std::string, std::string, std::string>>
+        tTransforms{
+            std::make_tuple(std::string("special imine r1"),
+                            std::string("[Cz0R0X4!H0]-[c]=[nz0]"),
+                            std::string(""), std::string("")),
+            std::make_tuple(std::string("special imine r2"),
+                            std::string("[Cz0R0X4!H0]-[c](=c)-[nz0]"),
+                            std::string("==-"), std::string("")),
+        };
+
+    params.tautomerTransformData = tTransforms;
+    MolStandardize::TautomerEnumerator te(params);
+
+    auto m1 = "c1ccc([C@@H](CC2=NCCN2)c2ccccn2)cc1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "C([C@H](C1=CC=CC=C1)C2=NC=CC=C2)C3=NCCN3"_smiles;
+    REQUIRE(m2);
+
+    std::cerr << " * - * - * - * m1" << std::endl;
+    std::unique_ptr<ROMol> res1{te.canonicalize(*m1)};
+    REQUIRE(res1);
+    std::cerr << " * - * - * - * m2" << std::endl;
+    std::unique_ptr<ROMol> res2{te.canonicalize(*m2)};
+    REQUIRE(res2);
+    CHECK(MolToSmiles(*res1) == MolToSmiles(*res2));
+  }
+}
