@@ -227,6 +227,7 @@ RWMol *toMol(const std::string &inp,
     if (!molVect.empty()) {
       res = molVect[0];
       SmilesParseOps::CloseMolRings(res, false);
+      SmilesParseOps::CheckChiralitySpecifications(res, true);
       SmilesParseOps::SetUnspecifiedBondTypes(res);
       SmilesParseOps::AdjustAtomChiralityFlags(res);
       // No sense leaving this bookmark intact:
@@ -408,6 +409,22 @@ RWMol *SmilesToMol(const std::string &smiles,
   res = toMol(lsmiles, smiles_parse, lsmiles);
   handleCXPartAndName(res, params, cxPart, name);
   if (res && (params.sanitize || params.removeHs)) {
+    if (res->hasProp(SmilesParseOps::detail::_needsDetectAtomStereo)) {
+      // we encountered a wedged bond in the CXSMILES,
+      // these need to be handled the same way they were in mol files
+      res->clearProp(SmilesParseOps::detail::_needsDetectAtomStereo);
+      if (res->getNumConformers()) {
+        const auto &conf = res->getConformer();
+        if (!conf.is3D()) {
+          MolOps::assignChiralTypesFromBondDirs(*res, conf.getId());
+        }
+      }
+    }
+    // if we read a 3D conformer, set the stereo:
+    if (res->getNumConformers() && res->getConformer().is3D()) {
+      res->updatePropertyCache(false);
+      MolOps::assignChiralTypesFrom3D(*res, res->getConformer().getId(), true);
+    }
     try {
       if (params.removeHs) {
         bool implicitOnly = false, updateExplicitCount = true;
@@ -420,14 +437,21 @@ RWMol *SmilesToMol(const std::string &smiles,
       delete res;
       throw;
     }
-    // figure out stereochemistry:
-    if (params.useLegacyStereo) {
-      bool cleanIt = true, force = true, flagPossible = true;
-      MolOps::assignStereochemistry(*res, cleanIt, force, flagPossible);
-    } else {
-      bool cleanIt = true, flagPossible = false;
-      Chirality::findPotentialStereo(*res, cleanIt, flagPossible);
+    if (res->hasProp(SmilesParseOps::detail::_needsDetectBondStereo)) {
+      // we encountered either wiggly bond in the CXSMILES,
+      // these need to be handled the same way they were in mol files
+      res->clearProp(SmilesParseOps::detail::_needsDetectBondStereo);
+      MolOps::clearSingleBondDirFlags(*res);
+      MolOps::detectBondStereochemistry(*res);
     }
+    // figure out stereochemistry:
+    bool oVal = Chirality::getUseLegacyStereoPerception();
+    if (!params.useLegacyStereo) {
+      Chirality::setUseLegacyStereoPerception(false);
+    }
+    bool cleanIt = true, force = true, flagPossible = true;
+    MolOps::assignStereochemistry(*res, cleanIt, force, flagPossible);
+    Chirality::setUseLegacyStereoPerception(oVal);
   }
   if (res && res->hasProp(common_properties::_NeedsQueryScan)) {
     res->clearProp(common_properties::_NeedsQueryScan);
